// Copyright 2011 Google Inc. All Rights Reserved.

/**
 * @fileoverview The main script for the Clear History extension for Chrome.
 * @author arunjit@google.com (Arunjit Singh)
 */

function getClearDate(s) {
  var time = ~~(s.time) || CONSTANTS.DEFAULT_TIME;
  if (time == -1)
    return time;
  return (Date.now() - (time * 60 * 60 * 1000));  // time from hrs to ms
}

/**
 * Removes a single cookie
 * @param {Object.<Cookie>} cookie The cookie to delete.
 * @param {Function} callback The function to invoke when the cookie is deleted.
 */
function removeCookie(cookie, callback) {
  var url = 'http' + (cookie.secure ? 's' : '') + '://' + cookie.domain +
            cookie.path;
  chrome.cookies.remove({url: url, name: cookie.name}, callback);
}

/**
 * Removes all cookies
 * @param {Function=} callback The function to invoke when all cookies are
 *     deleted (optional).
 */
function clearCookies(callback, s) {
  callback = callback || function() {};
  chrome.cookies.getAll({}, function(cookies) {
    var len = cookies.length;
    // Synchronize the cookie deletion and execute the callback only when all
    // cookie deletion functions have called back (finished).
    (function doCookieCleanup(i) {
      if (i === len) {
        callback(s);
      } else {
        removeCookie(cookies[i], function() {
          doCookieCleanup(i + 1);
        });
      }
    })(0);
  });
}

/**
 * Clear download history
 */
function clearDownloads(s) {
  var query = {};
  var date = new Date(getClearDate(s));
  // chrome.downloads expects ISO 8601
  var iso8601 = date.toISOString();
  if (s.timeStart === CONSTANTS.YES)
    query.startedAfter = iso8601;
  else
    query.startedBefore = iso8601;
  chrome.downloads.erase(query);
}

/**
 * Callback for when history deletion is successful
 */
function didClearHistory(s) {
  var time = ~~(s.time) || CONSTANTS.DEFAULT_TIME;
  time = getUnitsForTime(time);
  var timeMsg = (time.time === -1) ? 'notificationTimeAll' :
                (s.timeStart === CONSTANTS.YES) ? 'notificationTimeNewer' :
                'notificationTimeOlder';
  var timeString = chrome.i18n.getMessage(timeMsg, [time.time, time.units]);
  var message = chrome.i18n.getMessage('notificationBody', timeString);
  var notification = webkitNotifications.createNotification(
      chrome.extension.getURL('/images/icon48.png'),
      chrome.i18n.getMessage('notificationTitle'),
      message);
  notification.show();
  setTimeout(function() {
    notification.cancel();
  }, 5000);
}

/**
 * Clears the history for the amount of time stored in {@code localStorage}
 */
function clearHistory(s) {
  //Get the values from localStorage
  // time is in hours
  var callback = function() { didClearHistory(s); };
  var time = getClearDate(s);
  if (time === -1) {
    // Delete everything
    chrome.history.deleteAll(callback);
  } else {
    // Create the range
    var startTime = time;
    if (s.timeStart === CONSTANTS.YES) {
      var endTime = Date.now();
    } else {
      var endTime = time;
      time = 0;
    }
    var range = {
      startTime: startTime,
      endTime: endTime
    };
    // Delete history in the range
    chrome.history.deleteRange(range, callback);
  }
}

/**
 * Run through all the clearing funcs
 */
function clearAll(s) {
  if (s.downloads === CONSTANTS.YES)
    clearDownloads(s);
  clearHistory(s);
}

function runCleaner() {
  var keys = [
    'cookies', 'downloads', 'time', 'timeStart',
  ];
  chrome.storage.sync.get(keys, function(s) {
    if (s.cookies === CONSTANTS.YES) {
      clearCookies(clearAll, s);
    } else {
      clearAll(s);
    }
  });
}

/**
 * Executes when the user clicks the browser action. Uses stored values from
 * {@code localStorage}
 */
chrome.browserAction.onClicked.addListener(function(tab) {
  // Get the value from localStorage
  chrome.storage.sync.get('prompt', function(s) {
  var showPrompt = s.prompt || CONSTANTS.YES;

  // The confirmation message to ask
  var message = chrome.i18n.getMessage('confirmPrompt');

  // Clear cookies, and then clear the history
  if (showPrompt === CONSTANTS.YES) {
    if (!confirm(message))
      return;
  }
  runCleaner();
  });
});

chrome.alarms.get(CONSTANTS.CLEANER_ALARM, function(a) {
  if (a)
    return;

  chrome.alarms.create(CONSTANTS.CLEANER_ALARM, {
    // First fire 10 minutes from now.
    'when': Date.now() + (10 * 60 * 1000),
    'periodInMinutes': 24 * 60
  });
});

chrome.alarms.onAlarm.addListener(function(alarm) {
  chrome.storage.sync.get('autoclear', function(s) {
    var autoCleaner = s.autoclear || CONSTANTS.NO;
    if (autoCleaner === CONSTANTS.YES)
      runCleaner();
  });
});
